import { Server } from 'socket.io'
import { TSocket } from '../../utilities/types'
import ZoneRepository from '../../repositories/zoneRepository'
import { Character, Zone } from '@prisma/client'
import CharacterManager from '../../managers/characterManager'
import { gameLogger } from '../../utilities/logger'

interface IResponse {
  zone: Zone
  characters: Character[]
}

export default class CharacterJoinEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('zone:character:join', this.handleCharacterJoin.bind(this))
  }

  private async handleCharacterJoin(callback: (response: IResponse) => void): Promise<void> {
    try {
      if (!this.socket.characterId) return

      const character = CharacterManager.getCharacterFromSocket(this.socket)
      if (!character) return

      const zone = await ZoneRepository.getById(character.zoneId)
      if (!zone) {
        gameLogger.error('zone:character:join error', 'Zone not found')
        return
      }

      if (character.zoneId) {
        this.socket.leave(character.zoneId.toString())
        this.io.to(character.zoneId.toString()).emit('zone:character:leave', character)
      }

      this.socket.join(zone.id.toString())

      // let other clients know of new character
      this.io.to(zone.id.toString()).emit('zone:character:join', character)

      // send over zone and characters to socket
      callback({ zone, characters: CharacterManager.getCharactersInZone(zone) })
    } catch (error: any) {
      gameLogger.error('zone:character:join error', error.message)
      this.socket.disconnect()
    }
  }
}